// =========================================
// Demo file for the DF3 clouds created with the makecloud2 macro
// -----------------------------------------
// Copyright Gilles Tran May 2003 http://www.oyonale.com
// -----------------------------------------
// Feel free to use and modify
// -----------------------------------------
// Render this image with a 1.33 ratio such as 800*600
// -----------------------------------------
// Render stats on a Pentium 4 1.7 Gz, 1 Gb Ram
// 800 * 600 +A0.3
// Parsing time 37s
// Render time 34h
// RAM 182 Mb
// -----------------------------------------
// This scene renders much slowly than the other demo scenes, possibly because it uses two
// light sources for the clouds
// -----------------------------------------

#version 3.7;
#include "colors.inc"
#include "functions.inc"
#include "transforms.inc"

//---------------------------------------
// Switches
//---------------------------------------
#declare RadOK=1; // radiosity off=0/on=1
#declare CloudOK=1; // clouds on=1
#declare TerrainOK=2; // low-res=0 hi-res=1 very high-res=2
#declare TerrainSmoothOK=1; // smooth terrain on=1
#declare WoodsOK=0; // woods on=1
#declare Intervals = 7; // cloud media intervals 1 for tests and higher values (2-5) for smoother results  //________________________________________________________
#declare Turbulence=0.15;//0.15; // cloud turbulence
#declare Lambda=3; // cloud lambda
#declare Omega=0.6; // cloud omega


//---------------------------------------
// Settings
//---------------------------------------
global_settings {
    assumed_gamma 2.2
    max_trace_level 256
    #if (RadOK=1)
        radiosity {
            count 10
            recursion_limit 1
            low_error_factor .5 
            gray_threshold 0.0  
            minimum_reuse 0.015 
            brightness 1
            adc_bailout 0.01/2
            brightness 0.5
            normal on
//            media on
        }
    #end
}

//---------------------------------------
// Camera
//---------------------------------------
camera {
  location  <0, 0, 0>
  direction z
  sky y+x*0.02 // slight tilt of the camera!
  right     -image_width/image_height*x
  look_at   <0,150,1000>
  rotate x*-5
  translate y*1000*0.55
}
#declare Pos_Camera=y*1000;
//---------------------------------------
// Common stuff
//---------------------------------------
#declare C_Sun=<184,162,124>/255;
#declare C_SkyTop=rgb <34,49,80>/255;
#declare C_Cloud=rgb <112,107,101>/255;

#declare C_Sky1=rgb <179,181,192>/255;
#declare C_Sky2= rgb C_Sun;
#declare C_Sky3= rgb <112,140,221>/255;

#declare C_Sand=rgb <105,126,112>/255;
#declare C_Sand2=rgb <70,79,61>/255;

#declare C_SkyTop =     rgb <153,172*1.06,190*1.06>/255;
#declare C_SkyBottom =  rgb <123/1.06,132*1.06,139*1.06>/209;
//#declare C_Sun =      rgb <123,132*0.75,139*0.5>/100;
#declare C_Cloud =          <0.85,0.85,0.85>;
#declare C_Sand =       rgb <105,126,112>*0.5/255;
#declare C_Sand2 =      rgb <70,79,61>/255;
#declare C_SkyBlue_05 = rgb <209,224,183>/255-(<234,255,160>/255-<209,224,183>/255)*0.5;
#declare C_SkyBlue_06 = rgb <234,255,160>/255+(<234,255,160>/255-<209,224,183>/255)*0.5;
#declare C_Sun = rgb <255,174,37>/255*0.5;

#declare C_Scattering=C_Cloud*0.01;
#declare C_Absorption=(1-C_Cloud)*0.002;
#declare C_Emission=C_Sky3*0.0004;
#declare C_Fog=C_Sky2*1.2;
#declare T_Clear=texture{pigment{Clear}finish{ambient 0 diffuse 0}}

#macro GammaColor(Color,Gamma)
        rgb <pow(Color.red,Gamma),pow(Color.green,Gamma),pow(Color.blue,Gamma)>
#end

//---------------------------------------
// Sun and sky
//---------------------------------------
// This is sun for the clouds
// The terrain has its own light group with the sun slightly higher above the horizon
#declare SunAngleX       = 14;
#declare SunAngleX_Terrain = 12;
#declare SunAngleY       = -90; 
#declare Transform_Sun=transform{
    rotate x*SunAngleX
    rotate y*SunAngleY
}
#declare Pos_Sun=(vaxis_rotate(vaxis_rotate(vaxis_rotate(z,x,-SunAngleX),y,SunAngleY-180)*<1,1,2>,x,-0.5)-z)*100000000;

// main sun
light_source {
    -z*100000
    color GammaColor(C_Sun,2)
    transform{Transform_Sun}
}

// filler light coming from above
// it's there to get more volume to the clouds
// I suspect that it causes the big difference in render time

light_source {
    -z*100000
    color C_Sun*0.1
    rotate x*80
}

#declare P_SkySun=pigment {
        pigment_pattern{
            cylindrical
            rotate x*90     
            scale 0.6
            rotate x*-SunAngleX
            rotate y*(SunAngleY-180)
        }
        pigment_map{
            [ 0 function {min(1,max(0,y))} 
                poly_wave 1
                color_map {
                    [0.0 White]
                    [0.5 C_Sky2*1.3]
                    [1 C_Sky3*0.8]
                }
                scale 0.32
            ]
            [ 0.95 color White]
            [ 1 color C_Sun*10]
        }
    
    }

#declare P_SkyBack=pigment {
    function {min(1,max(0,y))}
    poly_wave 0.6
    color_map{
        [0 color C_Fog]
        [1 color C_SkyTop]
    }
}
#declare Sky=sky_sphere {
    pigment {
        gradient z
        pigment_map{
            [0.5 P_SkyBack]
            [1 P_SkySun]
        }
        scale <1,1,2>
        rotate x*-0.5
        translate -z
    }
}
//sky_sphere{Sky}

sky_sphere {
        pigment {
                gradient y 
                color_map {
                        [0.05 C_SkyBlue_06]                        
                        [0.60 C_SkyBlue_05]
                }
        }
}

fog {C_SkyBlue_06 fog_type 2 distance 1000*2  fog_alt 300 }

//---------------------------------------
// Terrain
//---------------------------------------
/*
#if (TerrainOK=1)
    #declare Resolution=1000;
#else
    #if (TerrainOK=2)
        #declare Resolution=6000;
    #else
        #declare Resolution=300;
    #end        
#end
#debug "Terrain\n"
#declare P_Desert=pigment{
    crackle solid
    turbulence 0.3
    lambda 3
    scale <1,30,1>/2000
    color_map{
        [0 color rgb <132,134,118>*0.7/255]
        [0.5 color rgb <80,93,93>*0.7/255]
        [1 color rgb <131,132,118>*0.5/255]
    }            
}                               
#declare SC=2.2;
#declare P_Vegetation1=pigment{
    crackle solid
    turbulence 0.3
    lambda 3
    scale <1,30,1>/2000
    color_map{
        [0 color rgb <95,120,82>*SC/255]
        [0.3 color rgb <60,85,50>*SC/255]
//        [0.6 color rgb <32,44,30>*SC/255]
        [0.6 color rgb <133,142,49>/255]
        [1 color rgb <52,70,68>*SC/255]
    }
}
#declare SC=0.5;
#declare P_Vegetation2=pigment{
    crackle solid
    turbulence 0.3
    lambda 3
    scale <1,30,1>/2000
    color_map{
        [0 color rgb <62,85,40>*0.6*SC/255]
        [0.5 color rgb <92,120,82>*0.5*SC/255]
        [1 color rgb <84,110,57>*0.8*SC/255]
    }
}
#declare SC=0.6;
#declare P_Vegetation3=pigment{
    crackle solid
    turbulence 0.3
    lambda 3
    scale <1,30,1>/2000
    color_map{
        [0 color rgb <28,44,42>*0.5*SC/255]
        [0.5 color rgb <74,110,67>*0.8*SC/255]
        [1 color rgb <52,85,50>*0.6*SC/255]
    }
}
#declare rt = seed (157);       //84, 157, 196
#declare tfax = (0.5-rand (rt))*5;
#declare tfay = (0.5-rand (rt))*5;
#declare tfaz = (0.5-rand (rt))*5;  ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#declare TerrainBase=height_field{
    function Resolution, Resolution {f_ridged_mf ( x*2+tfax, y*2+tfay, z+tfaz,0.6, 3, 6, -1, 3,3 )*0.2}
    #if (TerrainSmoothOK=1)
        smooth
    #end
    translate -x*0.5
}
#declare Terrain= object{
    TerrainBase
    translate x*0.5 rotate y*-90 translate x*0.5 
    texture{ 
        pigment{
            pigment_pattern{
                gradient y
                turbulence 0.0001
                lambda 3
                omega 3
            }
            pigment_map{
                [0.4 color Black] // lake bottom
                [0.51 color C_Sand2]
                [0.515 color C_Sand2*0.2+<52,85,40>*0.2/255]
                [0.52 color C_Sand]
                [0.525 P_Desert]
                [0.53 
                    pigment_pattern{
                        crackle solid scale 1/40 turbulence 1
                    } 
                    pigment_map{
                        [0. P_Desert]
                        [0.21 P_Vegetation2]
                        [1 P_Vegetation1]
                    }
                ]
            }
        }
        normal{
            average
            normal_map{
                [1 
                    pigment_pattern{
                        agate
                        color_map{
                            [0.99 White]
                            [1 White*0.7]
                        } 
                    }
                    bump_size 0.5
                ]
                [1
                    granite 0.3
                ]
            }
            scale <1,30,1>/200
                accuracy 0.003
        }
        finish{ambient 0 diffuse 1 specular 0}
    }
}

#declare Terrain1=object{Terrain scale <35000,1000,35000>}

#declare Terrain2=object{
    TerrainBase
    pigment{Black} finish{ambient 0 diffuse 0}
    translate -y*0.2
    scale <30000,1000,30000>
}                          

//---------------------------------------
// Trees and woods
//---------------------------------------
#debug "Woods\n"

#macro Tree()
    // creates a media-filled sphere that looks like a tree 
    #local SC=rand(rd)+1;
    #local C_Media=<32+rand(rd)*30,44+rand(rd)*5,30>/255;
    sphere{0,2 
        texture{T_Clear}
        hollow
        interior{
            media{
                scattering {2,C_Media*(0.3+rand(rd)*0.3)*0.3 extinction 1}
                absorption (1-C_Media*(0.1+rand(rd)*0.4)*1)
                samples 1,1
                density{
                    spherical
                    turbulence 0.5+rand(rd)
                    lambda 2.5+rand(rd)
                    rotate y*360*rand(rd)
                    translate y*rand(rd)*0.5
                    
                }
            }
        }
        translate y*0.6 scale 2+rand(rd)*7 scale 1*<SC,0.5+rand(rd)*2,SC>
    }            
#end 

#declare rd=seed(0);

#if (WoodsOK=1)
    #declare Woods=union{
        #declare i=0;
        #while (i<3000)
            #declare Start=<-1000+rand(rd)*5000,1000,rand(rd)*4000+500>;
            #declare Norm=<0,0,0>;
            #declare SC=rand(rd)+1;
            #declare Inter= trace (Terrain1, Start, -y, Norm );
            #if (vlength(Norm)!=0 & Inter.y>520)
                #if (rand(rd)<0.2)
                    #declare j=0;
                    #declare Start0=Start;
                    #while (j<20+rand(rd)*20)
                        #if (rand(rd)<0.5) // lines of trees
                            #declare Dir=<int(rand(rd)*2)*2-1,0,int(rand(rd)*2)*2-1>*(1+rand(rd))*4;
                            #declare Start=Start+Dir;
                        #else // groves and thickets
                            #declare Start=vaxis_rotate((1+rand(rd))*20,y,360*rand(rd))+Start0;
                        #end
    
                        #declare Norm=<0,0,0>;
                        #declare Inter= trace (Terrain1, Start, -y, Norm );
    
                        #if (vlength(Norm)!=0 & Inter.y>520)
                            object{Tree() translate Inter}
                        #end
                        
                        #declare j=j+1;
                    #end
                #else // single tree
                    object{Tree() translate Inter}
                #end
    //            cylinder{0,y*10,3 pigment{Red} translate Inter}
            #end
            #declare i=i+1;
        #end
    }
#end      


//---------------------------------------
// Terrain placement
//---------------------------------------
// The terrain group has its own sun
// It's much brighter than the sun used for the clouds
// The position is a little higher too
#declare Transform_Sun_Terrain=transform{
    rotate x*SunAngleX_Terrain
    rotate y*SunAngleY
}
light_group{
    light_source {
        -z*100000
        color GammaColor(C_Sun,2)*30
        transform{Transform_Sun_Terrain}
    }
    union{
        #if (WoodsOK=1)
            object{Woods}
        #end
        object{Terrain1}
        translate -x*1000+z*400
    }
    object{Terrain2 rotate -z*0.1 scale 16 translate z*30000-y*6000*0.6-x*10000}
    
    // water
    plane{y,510 
        texture{
            #declare SC=1/255;
            pigment{rgbt <SC*57, SC*63, SC*66, 0.9>}
            normal{ 
                ripples 0.03
                frequency 500
                scale 7000
                turbulence 0.2
                lambda 2.1
            }
    
            finish{ambient 0 diffuse 0.3 reflection {0,1 fresnel on} conserve_energy}
        }
        interior{
            ior 1.34
            fade_distance 1
            fade_power 2
            fade_color rgbt <SC*37, SC*63, SC*66, 0.9>
        }

    }
    // bottom plane just in case
    difference{
        plane{y,-100}
        plane{z,0 inverse}
        texture{pigment{color rgb <154,158,134>*0.3/255} finish{ambient 0 diffuse 1}}
    }
    global_lights false
}*/
//---------------------------------------
// Clouds
//---------------------------------------
#declare rd=seed(10);

#if (CloudOK=1)
#debug "Clouds\n"
    #declare CutY=0.3;
    #declare Cloud1=box{-0.5,0.5 scale <1,(1-CutY),1>
        texture{pigment{Clear}finish{ambient 0 diffuse 0}}
//        texture{pigment{Red}finish{ambient 0 diffuse 1}}
        hollow
        interior{
            media{
                scattering{1,C_Scattering extinction 0.15}
                absorption C_Absorption
                emission C_Emission
                intervals Intervals
                density{
                    density_file df3 "cloud2_1.df3"
                    turbulence Turbulence
                    lambda Lambda
                    omega Omega
                    interpolate 1
                    translate -0.5
                    scale <1,-1,1>
                }
            }
        }
        scale <4/3,1,1>
        //rotate y*90
    }
    #declare Cloud2=box{-0.5,0.5
        texture{pigment{Clear}finish{ambient 0 diffuse 0}}
//        texture{pigment{Red}finish{ambient 0 diffuse 1}}
        hollow
        interior{
            media{
                scattering{1,C_Scattering extinction 0.15}
                absorption C_Absorption
                emission C_Emission
                intervals Intervals
                density{
                    density_file df3 "cloud2_2.df3"
                    turbulence Turbulence
                    lambda Lambda
                    omega Omega
                    interpolate 1
                    translate -0.5
                    scale <1,-1,1>
                }
            }
        }
        scale <4/3,1,1>
        //rotate y*90
    }

    #declare Cloud3=box{-0.5,0.5
        texture{pigment{Clear}finish{ambient 0 diffuse 0}}
//        texture{pigment{Red}finish{ambient 0 diffuse 1}}
        hollow
        interior{
            media{
                scattering{1,C_Scattering extinction 0.15}
                absorption C_Absorption
                emission C_Emission
                intervals Intervals
                density{
                    density_file df3 "cloud2_3.df3"
                    turbulence Turbulence
                    lambda Lambda
                    omega Omega
                    interpolate 1
                    translate -0.5
                    scale <1,-1,1>
                }
            }
        }
        scale <4/3,1,1>
        //rotate y*90
    }
    #declare Cloud4=box{-0.5,0.5
        texture{pigment{Clear}finish{ambient 0 diffuse 0}}
//        texture{pigment{Red}finish{ambient 0 diffuse 1}}
        hollow
        interior{
            media{
                scattering{1,C_Scattering extinction 0.15}
                absorption C_Absorption
                emission C_Emission
                intervals Intervals
                density{
                    density_file df3 "cloud2_4.df3"
                    turbulence Turbulence
                    lambda Lambda
                    omega Omega
                    interpolate 1
                    translate -0.5
                    scale <1,-1,1>
                }
            }
        }
        scale <4/3,1,1>
    }
//---------------------------------------
// Cloudscape
//---------------------------------------
    union{
        
        object{Cloud2 scale <1,1,1> scale 7000 translate <2500,-500,7000>}
        
        object{Cloud2 scale <1,1,1> scale 7000 translate <-6000,0,17000>}
        object{Cloud2 scale <-1,1,1> scale 7000 translate <-13000,0,25000>}
        object{Cloud2 scale <-1,-1,1> translate -y*0.23 scale 7000 translate <8000,0,22000>}
        union{
            object{Cloud3 scale <-1,1,-1> scale 7000 translate <-20000,0,-2000>}
            object{Cloud4 scale <1,1,1> scale 8000 translate <-1000,0,-1000>}
            object{Cloud1 scale <1,1,1> scale 16700 translate <8000,0,3660>}
            object{Cloud3 scale <-1,1,1> scale 6300 translate <19500,0,3200>}
            translate z*45000
            scale <-1,1,1>
        }
        
        union{
            object{Cloud2 scale <-1,1,-1> scale 7000 translate <-30000,0,-2000>}
            object{Cloud3 scale <1,1,1> scale 8000 translate <-10000,0,-1000>}
            object{Cloud4 scale <1,-1,-1> scale 8000 translate <2000,0,1000>}
            object{Cloud1 scale <1,1,1> scale 16700 translate <18000,0,3660>}
            object{Cloud4 scale <-1,1,1> scale 6300 translate <38000,0,-3200>}
            translate z*65000
            scale <1,1,1>
        }
        
        translate y*4000
    }
#end
